import random, logging
from mashbase import MashBase

class ContextualMash(MashBase):
    """This class will be the parent class that all mash classes inherit from"""
    def __init__(self, debug = logging.CRITICAL):
        self.logger = logging.getLogger('ContextualMasher')
        logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.logger.setLevel(debug)
        
        #A list of words that each word can merge with
        self.available_mashes = {
             'seven': ['oven', 'spoon'],
             'coach': ['car_gray'],
             'pot': ['boat_plate', 'boat', 'plate', 'white'],
             'one': ['van_wagon', 'wagon'],
             'four': ['chair_fairy', 'fairy', 'fairy_friday', 'fork', 'friday'],
             'zero': ['chair'],
             'tissue': ['bus', 'three', 'tuesday'],
             'oven': ['seven', 'wagon'],
             'pen': ['plane', 'spoon'],
             'table': ['blue'],
             'ship': ['chair', 'soap'],
             'chair': ['car', 'car_gray', 'fairy_four', 'gray', 'ship', 'zero'],
             'boat': ['eight', 'plate', 'plate_pot', 'pot'],
             'blue': ['black', 'bowl', 'plane', 'plane_plate', 'plate', 'table'],
             'van': ['one_wagon', 'wagon'],
             'nine': ['green', 'plane'],
             'bowl': ['blue'],
             'three': ['car', 'fairy', 'tissue', 'train', 'tram', 'truck'],
             'thursday': ['tuesday'],
             'sunday': ['monday'],
             'bike': ['black'],
             'black': ['bike', 'blue', 'plane_plate'],
             'teaspoon': ['spoon'],
             'green': ['gray', 'nine', 'train'],
             'white': ['eight', 'pot'],
             'stove': ['glove', 'two'],
             'red': ['friday'],
             'fork': ['four', 'friday', 'truck'],
             'plate': ['black_plane', 'blue_plane', 'blue', 'boat', 'boat_pot', 'plane', 'pot'],
             'monday': ['sunday'],
             'wagon': ['one_van', 'one', 'oven', 'van'],
             'bus': ['tissue'],
             'friday': ['fairy_four', 'fairy', 'fork', 'four', 'red'],
             'glass': ['glove'],
             'spoon': ['pen', 'seven', 'soap', 'teaspoon'],
             'plane': ['black_plate', 'blue', 'blue_plate', 'nine', 'pen', 'plate'],
             'eight': ['boat', 'white'],
             'two': ['stove'],
             'gray': ['car_chair', 'car_coach', 'car', 'chair', 'green'],
             'tram': ['three', 'train'],
             'car': ['chair', 'chair_gray', 'coach_gray', 'gray', 'three'],
             'glove': ['glass', 'stove'],
             'soap': ['ship', 'spoon'],
             'train': ['green', 'three', 'tram', 'truck'],
             'truck': ['fork', 'three', 'train'],
             'fairy': ['chair_four', 'four', 'four_friday', 'friday', 'three'],
             'tuesday': ['thursday', 'tissue']
        }
        #A dictionary of word merges for a specific set of words (the key is the words in the merge joined together by a underscore)
        self.wordsdict =  {
             'car_three': ['chree'],
             'chair_gray': ['ghair'],
             'one_van_wagon': ['waon'],
             'fairy_friday': ['friaay', 'friiay', 'ferdy'],
             'car_chair_gray': ['cair'],
             'train_truck': ['traik'],
             'boat_pot': ['ptt', 'paot', 'bot', 'pbot', 'paat', 'pott', 'pboat', 'bat', 'btt', 'bott', 'pot', 'boat', 'pat', 'poat', 'pboot', 'boot', 'poot'],
             'soap_spoon': ['spooo', 'spoo'],
             'car_coach_gray': ['coarh'],
             'monday_sunday': ['munday', 'sonday'],
             'spoon_teaspoon': ['tspoon'],
             'three_train': ['traie', 'trai'],
             'fork_friday': ['frikay'],
             'car_chair': ['caair', 'ccair', 'hchar'],
             'blue_plane_plate': ['plaue', 'plae'],
             'oven_wagon': ['wavon'],
             'blue_bowl': ['bwlue'],
             'tissue_tuesday': ['tuesiay', 'tuessay', 'tuesay', 'tistue', 'tissdue'],
             'black_plane_plate': ['plake'],
             'blue_plate': ['blute', 'bltue'],
             'bus_tissue': ['bissue'],
             'gray_green': ['greey', 'gree', 'greny', 'greyn', 'grney'],
             'friday_red': ['riday'],
             'boat_eight': ['bight'],
             'three_tram': ['threm'],
             'three_tissue': ['thsee'],
             'ship_soap': ['shoap', 'siap'],
             'boat_plate': ['bloat'],
             'seven_spoon': ['svoon'],
             'bike_black': ['biack', 'blike', 'back'],
             'pen_plane': ['pane', 'peane'],
             'boat_plate_pot': ['poate'],
             'glove_stove': ['slove', 'gtove'],
             'nine_plane': ['pnane'],
             'oven_seven': ['eeven', 'even', 'oeven', 'soven'],
             'eight_white': ['whhte', 'wiiht', 'ihht', 'ehhte', 'weight', 'eiht', 'iht', 'whitt', 'eight', 'ihte', 'eiit', 'eihht', 'eit', 'ieht', 'wihht', 'ight', 'eitt', 'eite', 'weite', 'wiht', 'whte', 'ehht', 'eht', 'witt', 'wite', 'ehte'],
             'black_blue': ['bluke', 'bluek', 'blcue'],
             'fairy_four_friday': ['friay'],
             'train_tram': ['traim', 'tramn'],
             'green_nine': ['gneen'],
             'fairy_four': ['farry', 'fayr', 'foiry', 'foiur', 'faru', 'fairo', 'fairr', 'fuiry', 'ffry', 'faur', 'fauro', 'faury', 'ffaur', 'fair', 'ffour', 'fiir', 'ffir', 'foaur', 'foury', 'firy', 'fur', 'foarr', 'fouyr', 'fuir', 'fiur', 'fory', 'faar', 'fury', 'foir'],
             'fork_four': ['fourk'],
             'three_truck': ['threk', 'threc'],
             'stove_two': ['ttove', 'tove', 'wtove'],
             'fork_truck': ['fruck'],
             'one_wagon': ['waoon', 'agne', 'oagn', 'agn'],
             'pen_spoon': ['ppoon', 'poon'],
             'chair_ship': ['chaip'],
             'car_gray': ['cray', 'car', 'carr', 'crr', 'gray', 'cry', 'cra', 'gayr', 'caar', 'gar', 'cgar', 'gry', 'ggar', 'gaar', 'grr'],
             'plate_pot': ['pate'],
             'van_wagon': ['vagn', 'vgn'],
             'chair_zero': ['czero', 'czr', 'czaro', 'czer', 'czeir', 'ceair', 'czaer', 'czairo', 'cziro'],
             'blue_plane': ['blnue', 'blune'],
             'pot_white': ['wpite'],
             'glass_glove': ['glose'],
             'plane_plate': ['plane', 'plate', 'plale', 'plaee', 'platne', 'plaae', 'plape'],
             'green_train': ['treen', 'grain'],
             'four_friday': ['frioay'],
             'fairy_three': ['fhree'],
             'blue_table': ['tblue'],
             'chair_fairy_four': ['fhair'],
             'thursday_tuesday': ['thuasday', 'tuersday', 'thusday', 'truesday', 'thuesday']
        }

    def mash_word(self, word, predictionMatrix):
        """Mashes a word to the best possible combination of words"""
        if word in self.available_mashes:
            best = -1
            bestIndex = -1
            for i in range(0,len(self.available_mashes[word])):
                potential_words = self.available_mashes[word][i].split("_")
                total = 0.0
                for potential_word in potential_words:
                    if potential_word in predictionMatrix:
                        total += predictionMatrix[potential_word]
                if total > best:
                    bestIndex = i
                    best = total
            if bestIndex != -1:
                mashed_words = self.available_mashes[word][bestIndex].split("_") #Build the key
                mashed_words.append(word)
                mashed_words.sort()

                mashes = self.wordsdict["_".join(mashed_words)] #Get the list of mashes
                self.logger.debug("Using mash of (%s) to improve certainness by %f", " ".join(mashed_words), best)
                return mashes[random.randint(0, len(mashes) - 1)] #Pick one randomly
        return word















