%% Setup the parameters you will use for this exercise
input_layer_size  = 2048;
hidden_layer_size = 4096;
num_labels = 59;          % 58 labels, from 1 to 59 (inclusive, 0 is unused)

% Load Training Data
% X, each row consists of pixel values for a single input
X = csvread("trainingsetx.csv");
% m, number of samples
m = size(X, 1);
% y, each row is the index of the word which corresponds to the example in X
y = csvread("trainingsety.csv");

% Predict values for our validation set so we can see how good the algorithm actually is
Xvalidation = csvread("validationsetx.csv");
% y, each row is the index of the word which corresponds to the example in X
yvalidation = csvread("validationsety.csv");

inputs = [];
results = [];
accuracy = [];

for i = 1:5
	hidden_layer_size = i*512;
	% Initialize theta values
	% Theta1, takes us from the input layer to the hidden layer
	initial_Theta1 = randInitializeWeights(input_layer_size, hidden_layer_size);
	% Theta2, takes us from the hidden layer to the output layer
	initial_Theta2 = randInitializeWeights(hidden_layer_size, num_labels);
	% Unroll parameters
	initial_nn_params = [initial_Theta1(:) ; initial_Theta2(:)];

	% Train algorith with the training set
	options = optimset('MaxIter', 500);
	lambda = 0;
	% Create "short hand" for the cost function to be minimized
	costFunction = @(p) nnCostFunction(p, ...
	                                   input_layer_size, ...
	                                   hidden_layer_size, ...
	                                   num_labels, X, y, lambda);
	% Now, costFunction is a function that takes in only one argument (the
	% neural network parameters)
	[nn_params, cost] = fmincg(costFunction, initial_nn_params, options);
	% Obtain Theta1 and Theta2 back from nn_params
	Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
	                 hidden_layer_size, (input_layer_size + 1));
	Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
	                 num_labels, (hidden_layer_size + 1));

	% Use these theta values to make predictions based on the validation set
	pred = predict(Theta1, Theta2, Xvalidation);
	
	% save the input value, final cost, and final accuracy on the validation set
	inputs(i) = i*512
	results(i) = cost(length(cost))
	accuracy(i) = (mean(double(pred == yvalidation)) * 100)
end

% Finally, we print out the results and see what number of hidden nodes had the lowest cost
inputs
results
accuracy

