#!/bin/bash
# This will generate the input files for octave based on the values in ./dat/
#

# Clean up old output files
rm trainingset[xy].csv* y_mappings.txt 2> /dev/null
echo -n "Generating inputs for machine learning..."

i=1
# copy data from dat/${word} to trainingset[xy].csv and add y_mappings
for file in `ls dat/`; do
	word=${file%%.*}
	cat dat/$file >> trainingsetx.csv
	echo "$i, $word" >> y_mappings.txt
	line_count=`wc -l dat/$file | sed 's/ .*//g'`
	for j in `seq 1 $line_count`; do
		echo "$i" >> trainingsety.csv
	done
	i=$((i+1))
	echo -n "."
done
echo ""

# If each row isn't the same number of entries, we need to split them into
# three sections.
multiple_sections=`cat trainingsetx.csv | awk -F , 'BEGIN { w=0; f=0 } { if(w!=0 && w!=NF){f=1}; if(w==0){w=NF;} } END { if(f==1){print "True";}; }'`
if [[ "$multiple_sections" == "True" ]]; then
	# Not the most efficient way, but simplist to code
	echo "Splitting segments into files based on length"
	./split_into_3.py trainingsetx.csv trainingsety.csv
	
	# Screw randomizing and splitting into training/validation, we'll just
        # throw everything in training and then see how live testing goes for
        # the validation part!  Since we're not splitting into training/validation,
        # we don't need to reandomize the order of the training set.
	echo "Copying files into ../image-neural-network/"
	cp -v trainingsetx_[123].csv trainingsety_[123].csv y_mappings_[123].txt ../image-neural-network/
else
	# our records were ordered, but we want them in random order so we can split
	# our dataset into a training set and a validation set for analysis.  (We
	# can use the full dataset for training once we figure out all the parameters)
	./mushit.py trainingsetx.csv trainingsety.csv

	record_count=`wc -l trainingsetx.csv_mushed.csv | sed 's/^ *\([^ ]*\).*/\1/'`
	training_size=$((record_count*7/10))
	validation_size=$((record_count-training_size))

	echo "record_count = $record_count"
	echo "training_size = $training_size"
	echo "validation_size = $validation_size"

	head -n $((training_size)) trainingsetx.csv_mushed.csv > trainingsetx.csv
	head -n $((training_size)) trainingsety.csv_mushed.csv > trainingsety.csv
	tail -n $((validation_size)) trainingsetx.csv_mushed.csv > validationsetx.csv
	tail -n $((validation_size)) trainingsety.csv_mushed.csv > validationsety.csv

	# Awesome, we have all our files ready to rock, now it's time to copy them
	# over to the neural network folder for the next step.
	#cp trainingsetx.csv trainingsety.csv validationsetx.csv validationsety.csv ../image-neural-network/
fi

