#!/usr/bin/python
# This is used to take training sets generated by generate_ml_inputs.sh
# and split them into three different groups based on the length.
#
# This program assumes that y_mappings.txt contains the list which maps
# y values (numbers) to the words they represent.
# 
# Sample usage: ./split_into_3.py trainingsetx.csv trainingsety.csv
#
from optparse import OptionParser
import sys, random

def read_file(filename):
    f = open(filename)
    contents = f.readlines()
    f.close()
    return contents

def write_file(filename, contents):
    print "Writing out file: " + filename
    f = open(filename, "w")
    for line in contents:
        f.write(line + "\n")
    f.close()

def write_y_mappings(filename, keys, y_mappings):
    f = open(filename, "w")
    for key in keys:
        f.write(key + ", " + y_mappings[key] + "\n")
    f.close()

if __name__ == "__main__":
    parser = OptionParser(usage="%prog <input_x_values> <input_y_values>")
    (options, args) = parser.parse_args()
    
    if len(args) < 2:
        parser.print_help()
        print "ERROR: Filenames not specified"
        sys.exit(-1)    
    
    input_x = read_file(args[0])
    input_y = read_file(args[1])
    temp = [x.strip().split(", ") for x in read_file("y_mappings.txt")]
    y_mappings = {}
    for i,w in temp:
        y_mappings[i] = w
    
    if len(input_x) != len(input_y):
        print "Input files are not the same size"
        print len(contentsx),len(contentsy)
        sys.exit()
    print len(input_y), "Samples Loaded."
    
    output_x_1_filename = args[0].replace(".csv", "_1.csv")
    output_x_2_filename = args[0].replace(".csv", "_2.csv")
    output_x_3_filename = args[0].replace(".csv", "_3.csv")
    output_y_1_filename = args[1].replace(".csv", "_1.csv")
    output_y_2_filename = args[1].replace(".csv", "_2.csv")
    output_y_3_filename = args[1].replace(".csv", "_3.csv")
    data_x_1 = []
    data_x_2 = []
    data_x_3 = []
    data_y_1 = []
    data_y_2 = []
    data_y_3 = []
    for i in range(0, len(input_x)):
        values = input_x[i].strip().split(",")
        if len(values) == 2048:
            data_x_1.append(input_x[i].strip())
            data_y_1.append(input_y[i].strip())
        elif len(values) == 4096:
            data_x_2.append(input_x[i].strip())
            data_y_2.append(input_y[i].strip())
        elif len(values) == 6144:
            data_x_3.append(input_x[i].strip())
            data_y_3.append(input_y[i].strip())
    
    print "Writing out files"
    # Now we need to update the y_mappings for the small datasets
    y_keys_1 = list(set(data_y_1))
    y_keys_1.sort(lambda a,b: cmp(int(a), int(b)))
    write_y_mappings("y_mappings_1.txt", y_keys_1, y_mappings)
    
    y_keys_2 = list(set(data_y_2))
    y_keys_2.sort(lambda a,b: cmp(int(a), int(b)))
    write_y_mappings("y_mappings_2.txt", y_keys_2, y_mappings)
    
    y_keys_3 = list(set(data_y_3))
    y_keys_3.sort(lambda a,b: cmp(int(a), int(b)))
    write_y_mappings("y_mappings_3.txt", y_keys_3, y_mappings)
    
    # Finally, we write everything out to disk
    write_file(output_x_1_filename, data_x_1)
    write_file(output_y_1_filename, data_y_1)
    write_file(output_x_2_filename, data_x_2)
    write_file(output_y_2_filename, data_y_2)
    write_file(output_x_3_filename, data_x_3)
    write_file(output_y_3_filename, data_y_3)

