import sys, subprocess, Image

class SplitterBase():
    """This class will be the parent class that all splitters inherit from"""

    distance = 100 #How far away a color has to be from another color in order for it to not be considered the same color
    def __init__(self):
        return

    def split_captcha(self, filename):
        """Given the mp3 of an audio captcha, this class splits the file up into the individual words"""
        print "This method has not been implemented yet."
        sys.exit()

    def get_spectrogram(self, filename):
        """Get a spectrogram using sox"""
        p = subprocess.Popen(["sox", self.filename_prefix + filename + ".mp3", "-c", "1", "-n", "spectrogram", 
                                                "-o", self.filename_prefix + filename + ".png"], 
                                stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        p.wait()

    def get_duration(self, audiofile):
        """Gets the number of frames for the specified audio file"""
        p = subprocess.Popen(["sox", audiofile, "-c", "1", "-n", "stat"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output = p.communicate()[1]
        sampleLine = output.split("\n")[0]
        return float(sampleLine.split(":")[1].replace(" ", ""))

    def within_distance(self, color1, color2):
        """Determines whether or not a color is close enough to another color to be considered the same color"""
        if type(color1) is tuple and type(color2) is tuple:
            for i in range(0,3):
                if abs(color1[i] - color2[i]) > self.distance:
                    return False
            return True

    def remove_white_border(self, image):
        """Removes the white border from sox's spectrogram (there really should be a sox option for this)"""
        pixels = image.load()
        sizex,sizey = image.size
        centerx = int(sizex / 2)
        centery = int(sizey / 2)
        border_color = (127,127,127)
        top = left = right = bottom = None

        for x in range(centerx, 0, -1): #Start at the center and move in each direction to look for the border color
            if self.within_distance(pixels[x,centery],border_color):
                left = x
                break
        for x in range(centerx, sizex):
            if self.within_distance(pixels[x,centery],border_color):
                right = x
                break
        for y in range(centery, 0, -1):
            if self.within_distance(pixels[centerx,y],border_color):
                top = y
                break
        for y in range(centery, sizey):
            if self.within_distance(pixels[centerx,y],border_color):
                bottom = y
                break

        if left == None or right == None or top == None or bottom == None:
            self.logger.warning("Could not remove the spectrogram border, assuming the image has already been cropped")
            return image

        box = (left + 1, top + 1, right - 1, bottom - 1)
        self.logger.debug("Removing spectrogram border by cropping image to " + str(box))
        return image.crop(box)

    def split_wav_file(self, column_xs, sizex, filename, suffix = ""):
        """Splits the corresponding wav file based on the columns found in the spectrogram.  The column x start and x end should be
             passed in a list of tuples in column_xs and they should not overlap.  The width of the image should be passed in sizex, and
             optionally a naming suffix for the split up wav files can be passed in suffix."""
        fullfile = self.filename_prefix + filename + ".mp3"
        soundsizex = self.get_duration(fullfile)
        sizex = float(sizex)
        pos = 0.0
        sound_columns = []
        processes = []
        for i in range(0,len(column_xs)):
            #Sox has the ability to split multiple parts at once, but it doesn't work (but I couldn't get it to work)
            options = ["sox", fullfile, self.filename_prefix + filename + "_" + str(i) + suffix + ".wav", "trim"] 

            skip = float(column_xs[i][0]) * (soundsizex/sizex) * (1.0 / self.sample_rate) #The number of seconds to skip
            length = float(column_xs[i][1] - column_xs[i][0] + 1.0) * (soundsizex / sizex) * (1.0 / self.sample_rate) #The number of seconds to read

            options.append(str(skip))
            options.append(str(length))
            processes.append(subprocess.Popen(options, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE))
            sound_columns.append([skip, skip + length])

        for i in range(0, len(processes)):
            processes[i].communicate() #Wait for the last sox process to end
        self.logger.debug("Sound Column x-coordinates are: " + str(sound_columns))
